// C functions to bypass Copper limitations:
// * Copper does not implement floats and doubles yet. These functions convert floats to ints.
// * Copper does not support variadic functions (except through genericity).
// * Access to GTK structure attributes.

#include <stdint.h>
#include <gtk/gtk.h>
#include "gdk/gdkx.h"

//[of]:Glib / GObject
gpointer g_object_new_0 (GType t) 
{ 
	return g_object_new(t, 0); 
}

GType	_g_object_type	(GObject* obj)
{
	return G_OBJECT_TYPE(obj); 
}

void g_object_set_float (gpointer object, const gchar *property_name, gint p)
{
	g_object_set (object, property_name, ((float)p / 100), NULL); 
}
//[cf]
//[of]:Cairo
void c_fillColor (cairo_t* cr, int x, int y, int w, int h, GdkRGBA* rgba)
{
	if( (w > 0) && (h > 0) )
	{
		gdk_cairo_set_source_rgba (cr, rgba);
		cairo_set_line_width(cr, 1);
		if( h == 1 )
			// ????
			cairo_rectangle (cr, (double)x, (double)y+0.5, (double)w, (double)h-1.0);
		else if( w == 1 )
			// ????
			cairo_rectangle (cr, (double)x+0.5, (double)y, (double)w-1.0, (double)h);
		else
			cairo_rectangle (cr, (double)x+0.5, (double)y+0.5, (double)w-1.0, (double)h-1.0);
		cairo_stroke_preserve (cr);
		cairo_fill(cr);
	}
}

void c_show_layout (cairo_t* cr, PangoLayout      *layout, int x, int y)
{
	cairo_move_to(cr, (double)x, (double)y);
	pango_cairo_show_layout (cr, layout);
}
//[cf]
//[of]:GDK
///////////////////////////////////////////////////////////////////////////////
// GDK events
///////////////////////////////////////////////////////////////////////////////
guint	gdk_event_button_button	(GdkEventButton* e) { return e->button; }
int	gdk_event_button_get_x	(GdkEventButton* e) { return (int)e->x; }
int	gdk_event_button_get_y	(GdkEventButton* e) {	return (int)e->y; }
int	gdk_event_motion_get_x	(GdkEventMotion* e) { return (int)e->x; }
int	gdk_event_motion_get_y	(GdkEventMotion* e) { return (int)e->y; }
int	gdk_event_scroll_get_x	(GdkEventScroll* e) { return (int)e->x; }
int	gdk_event_scroll_get_y	(GdkEventScroll* e) { return (int)e->y; }

///////////////////////////////////////////////////////////////////////////////
// gdk_selection_owner_get cannot be used to check if a window from
// another process owns the selection (it fails with the xwindow -> gdkwindow
// conversion)
///////////////////////////////////////////////////////////////////////////////
int check_selection_owner_get(GdkAtom atom)
{
	GdkDisplay *display;
	Atom xatom;
	Window xwindow;
	
	display = gdk_display_get_default();
	xatom = gdk_x11_atom_to_xatom_for_display(display, atom);
	xwindow = XGetSelectionOwner (gdk_x11_display_get_xdisplay(display), xatom);
	return xwindow != None;	
}	

///////////////////////////////////////////////////////////////////////////////
// Cairo
///////////////////////////////////////////////////////////////////////////////
void gdk_cairo_set_source_pixbuf_int (cairo_t* cr, const GdkPixbuf* pixbuf, gint x, gint y)
	{ gdk_cairo_set_source_pixbuf (cr, pixbuf, (double)x, (double)y); }

void cairo_set_source_surface_int (cairo_t* cr, cairo_surface_t* s, gint x, gint y)
	{ cairo_set_source_surface (cr, s, (double)x, (double)y); }

///////////////////////////////////////////////////////////////////////////////
// GdkRGBA
///////////////////////////////////////////////////////////////////////////////
void rgbaToGdk (GdkRGBA* c, int red, int green, int blue, int alpha)
{
	c->red	= (double)red	/ 255.0;
	c->green	= (double)green	/ 255.0;
	c->blue	= (double)blue	/ 255.0;
	c->alpha	= (double)alpha	/ 255.0;
}

void gdkToRgba (GdkRGBA* c, int* red, int* green, int* blue, int* alpha)
{
	*red	= (int)(c->red	* 255.0 + 0.5);
	*green	= (int)(c->green	* 255.0 + 0.5);
	*blue	= (int)(c->blue	* 255.0 + 0.5);
	*alpha	= (int)(c->alpha	* 255.0 + 0.5);
}
//[cf]
//[of]:GTK
///////////////////////////////////////////////////////////////////////////////
// Range
///////////////////////////////////////////////////////////////////////////////
void	gtk_range_set_increments_int	(GtkRange *range, gint step, gint page)	{ gtk_range_set_increments(range, (gdouble)step, (gdouble)page); }
void	gtk_range_set_range_int	(GtkRange *range, gint min, gint max)	{ gtk_range_set_range (range, (gdouble)min, (gdouble)max); }
void	gtk_range_set_value_int	(GtkRange *range, gint value)	{ gtk_range_set_value(range, (gdouble)value); }
gint	gtk_range_get_value_int	(GtkRange *range)	{ return (gint)gtk_range_get_value(range); }

///////////////////////////////////////////////////////////////////////////////
// GtkTreeViewColumn
///////////////////////////////////////////////////////////////////////////////
void gtk_tree_view_column_set_alignment_float (GtkTreeViewColumn *tree_column, gint xalign)
	{ gtk_tree_view_column_set_alignment (tree_column, ((float)xalign)/100); }

///////////////////////////////////////////////////////////////////////////////
// GtkLabel
///////////////////////////////////////////////////////////////////////////////
void gtk_label_set_xalign_int (GtkLabel* label, int align)
	{ gtk_label_set_xalign(label, (float)align/100); }

void gtk_label_set_yalign_int (GtkLabel* label, int align)
	{ gtk_label_set_yalign(label, (float)align/100); }

///////////////////////////////////////////////////////////////////////////////
// GtkStyleContext
///////////////////////////////////////////////////////////////////////////////
void gtk_style_context_get_1 (	GtkStyleContext*	context,
	GtkStateFlags	state,
	const gchar*	name, 
	void*	value)
{
	gtk_style_context_get (context, state, name, value, NULL);
}

void c_gtk_render_background  (GtkStyleContext* cx, cairo_t* cr, int x, int y, int w, int h)
{
	gtk_render_background (cx, cr, (double)x+0.5, (double)y+0.5, (double)w-1.0, (double)h-1.0);
}

///////////////////////////////////////////////////////////////////////////////
// GtkAdjustment
///////////////////////////////////////////////////////////////////////////////
GtkAdjustment* gtk_adjustment_new_int(	 gint value,
	 gint lower,
	 gint upper,
	 gint step_increment,
	 gint page_increment,
	 gint page_size)
{
	return gtk_adjustment_new(
		(gdouble)value,
		(gdouble)lower,
		(gdouble)upper,
		(gdouble)step_increment,
		(gdouble)page_increment,
		(gdouble)page_size);
}
						 
void	gtk_adjustment_clamp_page_int	(GtkAdjustment *adjustment, gint lower, gint upper)	{ gtk_adjustment_clamp_page(adjustment, (gdouble)lower, (gdouble)upper); }
gint	gtk_adjustment_get_value_int	(GtkAdjustment *adjustment)	{ return (gint)gtk_adjustment_get_value(adjustment); }
void	gtk_adjustment_set_value_int	(GtkAdjustment *adjustment, gint value)	{ gtk_adjustment_set_value(adjustment, (gdouble)value); }

///////////////////////////////////////////////////////////////////////////////
// GtkListStore
///////////////////////////////////////////////////////////////////////////////
void	gtk_list_store_set_1	(GtkListStore* store,GtkTreeIter* iter, int v1,gchar* v2,int v3) { gtk_list_store_set(store, iter, v1, v2, v3); }

///////////////////////////////////////////////////////////////////////////////
// GtkWidget
///////////////////////////////////////////////////////////////////////////////
GtkWidget*	gtk_widget_new_0	(GType t) { return gtk_widget_new(t, 0); }

///////////////////////////////////////////////////////////////////////////////
// Dialogs
///////////////////////////////////////////////////////////////////////////////
GtkWidget* gtk_message_dialog_new_nofmt(GtkWindow      *parent,
                                        GtkDialogFlags  flags,
                                        GtkMessageType  type,
                                        GtkButtonsType  buttons,
                                        const gchar    *message)
{
	return gtk_message_dialog_new (parent, flags, type, buttons, "%s", message);
}

GtkWidget* gtk_file_chooser_dialog_new_1(const gchar *title,
                                         GtkWindow *parent,
                                         GtkFileChooserAction action,
                                         const gchar *button_1,
                                         int response_1,
                                         const gchar *button_2,
                                         int response_2)
{
	return gtk_file_chooser_dialog_new(title,
                                       parent,
                                       action,
                                       button_1, response_1,
                                       button_2, response_2,
                                       (const gchar *)0);
}

//[of]:GtkCustom
//[of]:Header
#define GTK_TYPE_CUSTOM	(gtk_custom_get_type ())
#define GTK_CUSTOM(obj)	(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_CUSTOM, GtkCustom))
#define GTK_CUSTOM_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_CUSTOM, GtkCustomClass))
#define GTK_IS_CUSTOM(obj)	(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_CUSTOM))
#define GTK_IS_CUSTOM_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_CUSTOM))
#define GTK_CUSTOM_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_CUSTOM, GtkCustomClass))

typedef struct _GtkCustom	GtkCustom;
typedef struct _GtkCustomClass	GtkCustomClass;

struct _GtkCustom
{
	GtkContainer container;
};

struct _GtkCustomClass
{
	GtkContainerClass parent_class;
};

GType	gtk_custom_get_type	(void) G_GNUC_CONST;
GtkWidget*	gtk_custom_new	(void);
//[cf]

#define GTK_CUSTOM_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GTK_TYPE_CUSTOM, GtkCustomPrivate))

typedef struct _GtkCustomPrivate GtkCustomPrivate;

struct _GtkCustomPrivate
{
	GList*	children;
	gpointer	container;
};

// Define the GtkCustom type and inherit from GtkContainer
G_DEFINE_TYPE(GtkCustom, gtk_custom, GTK_TYPE_CONTAINER);

// custom container
// A simple container where size_request and size_allocate are left
// to the signal handlers

static void gtk_custom_class_init	(GtkCustomClass *klass);
static void gtk_custom_init	(GtkCustom *custom);
static void gtk_custom_realize	(GtkWidget *widget);
static void gtk_custom_size_allocate	(GtkWidget* widget, GtkAllocation* allocation);
static void gtk_custom_add	(GtkContainer *container, GtkWidget *widget);
static void gtk_custom_remove	(GtkContainer *container, GtkWidget *widget);
static void gtk_custom_forall	(GtkContainer *container, gboolean include_internals, GtkCallback callback, gpointer callback_data);
static void gtk_custom_get_preferred_width	(GtkWidget *widget, gint* minimal_width, gint* natural_width);
static void gtk_custom_get_preferred_height	(GtkWidget *widget, gint* minimal_height, gint* natural_height);
static GType gtk_custom_child_type	(GtkContainer *container);

extern gint container_get_preferred_width	(gpointer container);
extern gint container_get_preferred_height	(gpointer container);

///////////////////////////////////////////////////////////////////////////////
// Class Initialization
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_class_init (GtkCustomClass *klass)
{
	GtkWidgetClass *widget_class;
	GtkContainerClass *container_class;
	
	widget_class = GTK_WIDGET_CLASS(klass);
	widget_class->realize = gtk_custom_realize;
	widget_class->size_allocate = gtk_custom_size_allocate;
	widget_class->get_preferred_width = gtk_custom_get_preferred_width;
	widget_class->get_preferred_height = gtk_custom_get_preferred_height;
	
	container_class = GTK_CONTAINER_CLASS(klass);
	container_class->add = gtk_custom_add;
	container_class->remove = gtk_custom_remove;
	container_class->forall = gtk_custom_forall;
	container_class->child_type = gtk_custom_child_type;

	g_type_class_add_private(klass, sizeof(GtkCustomPrivate));
}

///////////////////////////////////////////////////////////////////////////////
// return child type
///////////////////////////////////////////////////////////////////////////////
static GType gtk_custom_child_type (GtkContainer *container)
{
	return GTK_TYPE_WIDGET;
}

///////////////////////////////////////////////////////////////////////////////
// initialize
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_init (GtkCustom *custom)
{
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(custom);
	gtk_widget_set_has_window(GTK_WIDGET(custom), TRUE);
	priv->children = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// instance creation
///////////////////////////////////////////////////////////////////////////////
GtkWidget* gtk_custom_new (void)
{
	return g_object_new(GTK_TYPE_CUSTOM, NULL);
}

///////////////////////////////////////////////////////////////////////////////
// realize
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_realize (GtkWidget *widget)
{
	GdkWindowAttr attributes;
	GtkAllocation allocation;
	gint attributes_mask;
	
	gtk_widget_set_realized (widget, TRUE);

	gtk_widget_get_allocation (widget, &allocation);
	
	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.x = allocation.x;
	attributes.y = allocation.y;
	attributes.width = allocation.width;
	attributes.height = allocation.height;
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.visual = gtk_widget_get_visual (widget);
	attributes.event_mask = gtk_widget_get_events (widget);
	attributes.event_mask |= GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_SCROLL_MASK ;
	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
	
	GdkWindow* window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
      gtk_widget_register_window (widget, window);
	gtk_widget_set_window(widget, window);
	
	//gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
	//gdk_window_set_back_pixmap (window, NULL, FALSE);
}

///////////////////////////////////////////////////////////////////////////////
// size_allocate
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_size_allocate (GtkWidget* widget, GtkAllocation* allocation)
{
	g_return_if_fail (GTK_IS_CUSTOM (widget));
	g_return_if_fail (allocation != NULL);
	
	gtk_widget_set_allocation (widget, allocation);
	
	if (gtk_widget_get_realized (widget))
	{
		if (gtk_widget_get_has_window (widget))
		{
			gdk_window_move_resize (gtk_widget_get_window (widget), allocation->x, allocation->y, allocation->width, allocation->height);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// add
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_add (GtkContainer *container, GtkWidget *widget)
{
	g_return_if_fail (GTK_IS_CUSTOM (container));
	g_return_if_fail (GTK_IS_WIDGET (widget));
	
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(container);
	
	priv->children = g_list_append (priv->children, widget);
	gtk_widget_set_parent (widget, GTK_WIDGET (container));

	if( gtk_widget_get_visible(widget) )
		gtk_widget_queue_resize(GTK_WIDGET(container));
}

///////////////////////////////////////////////////////////////////////////////
// remove
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_remove (GtkContainer *container, GtkWidget *widget)
{
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(container);
	
	GList* link = g_list_find(priv->children, widget);
	if( link )
	{
		gboolean was_visible = gtk_widget_get_visible (widget);
		gtk_widget_unparent (widget);
			
		priv->children = g_list_delete_link(priv->children, link);
			
		if( was_visible )
			gtk_widget_queue_resize (GTK_WIDGET (container));
	}
}

///////////////////////////////////////////////////////////////////////////////
// forall
///////////////////////////////////////////////////////////////////////////////
static void gtk_custom_forall (	GtkContainer*	container,
	gboolean	include_internals,
	GtkCallback	callback,
	gpointer	callback_data)
{
	g_return_if_fail (callback != NULL);
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(container);
	g_list_foreach(priv->children, (GFunc)callback, callback_data);
}

///////////////////////////////////////////////////////////////////////////////
// get preferred width
///////////////////////////////////////////////////////////////////////////////
static void child_get_preferred_width(GtkWidget* w, gpointer data)
{
	gint minw, natw;
	gtk_widget_get_preferred_width(w, &minw, &natw);
}

static void gtk_custom_get_preferred_width(GtkWidget* widget, gint* minimal_width, gint* natural_width)
{
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(widget);

	g_list_foreach(priv->children, (GFunc)child_get_preferred_width, NULL);

	gint w = container_get_preferred_width (priv->container);
	*minimal_width = *natural_width = w;
}

///////////////////////////////////////////////////////////////////////////////
// get preferred height
///////////////////////////////////////////////////////////////////////////////
static void child_get_preferred_height(GtkWidget* w, gpointer data)
{
	gint minh, nath;
	gtk_widget_get_preferred_height(w, &minh, &nath);
}

static void gtk_custom_get_preferred_height(GtkWidget* widget, gint* minimal_height, gint* natural_height)
{
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(widget);
	g_list_foreach(priv->children, (GFunc)child_get_preferred_height, NULL);
	gint h = container_get_preferred_height (priv->container);
	*minimal_height = *natural_height = h;
}

///////////////////////////////////////////////////////////////////////////////
// set container
///////////////////////////////////////////////////////////////////////////////
void gtk_custom_set_container (GtkContainer* container, gpointer c)
{
	GtkCustomPrivate* priv = GTK_CUSTOM_PRIVATE(container);
	priv->container = c;
}
//[cf]
//[cf]
