
#include <QApplication>
#include <QPushButton>
#include <QCheckBox>
#include <QLineEdit>
#include <QFrame>
#include <QTableWidget>
#include <QLabel>
#include <QComboBox>
#include <QListWidget>
#include <QHeaderView>
#include <QScrollBar>
#include <QTabWidget>
#include <QTreeView>
#include <QDialog>
#include <QMessageBox>
#include <QColorDialog>
#include <QFileDialog>
#include <QMenuBar>
#include <QMenu>
#include <QInputEvent>
#include <QMouseEvent>
#include <QClipboard>
#include <QMimeData>
#include <QTimer>
#include <QFont>
#include <QFontMetrics>
#include <QCursor>
#include <QPainter>

#include "qt.h"

extern "C" {

//[of]:Utility Functions
static const char* toUtf8 (QByteArray* array, const QString& str)
{
	*array = str.toUtf8();
	return array->constData();
}

//[of]:rectangle_t
struct rectangle_t
{
	int x;
	int y;
	int w;
	int h;

	void set(const QRect& rect)
	{
		x = rect.x();
		y = rect.y();
		w = rect.width();
		h = rect.height();
	}
};
//[cf]
//[of]:point_t
struct point_t
{
	int x;
	int y;

	void set(const QPoint& pt)
	{
		x = pt.x();
		y = pt.y();
	}
};
//[cf]
//[cf]

//[of]:QObject
void	QObject_delete	(QObject* obj)	{ delete obj; }

void QObject_setPeer(QObject* obj, void* peer)
{
	obj->setProperty("peer", QVariant((qulonglong)(size_t)peer));
}

void* QObject_peer(QObject* obj)
{
	QVariant v = obj->property("peer");
	if( v.isValid() )
		return (void*)((size_t)v.toULongLong());
	else
		return NULL;
}

void QObject_installEventFilter(QObject* obj, QObject* filterObj)
{
	obj->installEventFilter(filterObj);
}
//[cf]
//[of]:	QCoreApplication
void QCoreApplication_postEvent(QObject *receiver, QEvent *event) { QCoreApplication::postEvent(receiver, event); }//[cf]
//[of]:		QGuiApplication
QClipboard* QGuiApplication_clipboard() { return QGuiApplication::clipboard(); }
//[cf]
//[of]:			QApplication
QApplication*	QApplication_new	(int* argc, char **argv)	{ return new QApplication (*argc, argv); }
void	QApplication_beep	()	{ QApplication::beep(); }
QWidget*	QApplication_focusWidget	()	{ return QApplication::focusWidget(); }
QStyle*	QApplication_style	()	{ return QApplication::style(); }
void	QApplication_delete	(QApplication* app)	{ delete app; }
int	QApplication_exec	(QApplication* app)	{ return app->exec(); }
//[cf]
//[of]:	QWidget
QWidget*	QWidget_new	(QWidget* parent)	{ return new QWidget(parent); }
void	QWidget_show	(QWidget* widget)	{ widget->show(); }
void	QWidget_hide	(QWidget* widget)	{ widget->hide(); }
void	QWidget_setFocus	(QWidget* widget)	{ widget->setFocus(); }
int	QWidget_close	(QWidget* widget)	{ return widget->close(); }
void	QWidget_showMaximized	(QWidget* widget)	{ widget->showMaximized(); }
int	QWidget_isEnabled	(QWidget* widget)	{ return widget->isEnabled(); }
int	QWidget_isEnabledTo	(QWidget* widget, QWidget* ancestor)	{ return widget->isEnabledTo(ancestor); }
int	QWidget_isMaximized	(QWidget* widget)	{ return widget->isMaximized(); }
int	QWidget_isVisible	(QWidget* widget)	{ return widget->isVisible(); }
int	QWidget_isVisibleTo	(QWidget* widget, QWidget* ancestor)	{ return widget->isVisibleTo(ancestor); }
int	QWidget_x	(QWidget* widget)	{ return widget->x(); }
int	QWidget_y	(QWidget* widget)	{ return widget->y(); }
int	QWidget_width	(QWidget* widget)	{ return widget->width(); }
int	QWidget_height	(QWidget* widget)	{ return widget->height(); }
void	QWidget_move	(QWidget* widget, int x, int y)	{ widget->move(x, y); }
void	QWidget_resize	(QWidget* widget, int w, int h)	{ widget->resize(w, h); }
void	QWidget_removeAction	(QWidget* widget, QAction* action)	{ widget->removeAction(action); }
void	QWidget_addAction	(QWidget* widget, QAction* action)	{ widget->addAction(action); }
void	QWidget_insertAction	(QWidget* widget, QAction* before, QAction* action)	{ widget->insertAction(before, action); }
void	QWidget_grabMouse	(QWidget* widget)	{ widget->grabMouse(); }
void	QWidget_releaseMouse	(QWidget* widget)	{ widget->releaseMouse(); }
const QPalette*	QWidget_palette	(QWidget* widget)	{ return &widget->palette(); }
void	QWidget_update	(QWidget* widget)	{ widget->update(); }
void	QWidget_updateRect	(QWidget* widget, int x, int y, int w, int h)	{ widget->update(x, y, w, h); }
void	QWidget_contentsRect	(QWidget* widget, rectangle_t *r)	{ r->set(widget->contentsRect()); }
void	QWidget_mapToGlobal	(QWidget* widget, int x, int y, point_t* pt)	{ pt->set(widget->mapToGlobal(QPoint(x, y))); }
void	QWidget_mapFromGlobal	(QWidget* widget, int x, int y, point_t* pt)	{ pt->set(widget->mapFromGlobal(QPoint(x, y))); }
void	QWidget_setEnabled	(QWidget* widget, int en)	{ widget->setEnabled(en); }
void	QWidget_setFixedSize	(QWidget* widget, int w, int h)	{ widget->setFixedSize(w, h); }
void	QWidget_setGeometry	(QWidget* widget, int x, int y, int w, int h)	{ widget->setGeometry(x, y, w, h); }
void	QWidget_setMouseTracking	(QWidget* widget, int en)	{ widget->setMouseTracking(en); }
void	QWidget_setCusor	(QWidget* widget, QCursor* cursor)	{ widget->setCursor(*cursor); }
void	QWidget_setWindowTitle	(QWidget* widget, const char* title)	{ widget->setWindowTitle(QString(title)); }
void	QWidget_setAttribute	(QWidget* widget, Qt::WidgetAttribute attribute, int on)	{ widget->setAttribute(attribute, on); }
void	QWidget_setWindowIcon	(QWidget* widget, QIcon* icon)	{ widget->setWindowIcon(*icon); }
void 	QWidget_setFocusPolicy	(QWidget* widget, Qt::FocusPolicy policy)	{ widget->setFocusPolicy(policy); }
void 	QWidget_setFont	(QWidget* widget, QFont* font)	{ widget->setFont(*font); }
void	QWidget_setPalette	(QWidget* widget, QPalette* palette)	{ widget->setPalette(*palette); }
void	QWidget_setContentsMargins	(QWidget* widget, int left, int top, int right, int bottom)	{ widget->setContentsMargins(left, top, right, bottom); }
void	QWidget_setStyleSheet	(QWidget* widget, const char* styleSheet)	{ widget->setStyleSheet(QString(styleSheet)); }
void	QWidget_setAcceptDrops	(QWidget* widget, int on)	{ widget->setAcceptDrops(on); }
void	QWidget_setMinimumSize	(QWidget* widget, int minw, int minh)	{ widget->setMinimumSize(minw, minh); }
void 	QWidget_setAutoFillBackground	(QWidget* widget, int enabled)	{ widget->setAutoFillBackground(enabled); }

int QWidget_minimumWidthHint (QWidget* widget)
{
	QSize qsize = widget->minimumSizeHint(); 
	if( qsize.isValid() )
		return qsize.width();
	else
		return 0;
}

int QWidget_minimumHeightHint (QWidget* widget)
{
	QSize qsize = widget->minimumSizeHint(); 
	if( qsize.isValid() )
		return qsize.height();
	else
		return 0;
}
//[cf]
//[of]:		QAbstractButton
typedef void (*ClickedFunc)(QAbstractButton* button, int checked);

void QAbstractButton_connectClicked(QAbstractButton* button, ClickedFunc fn)
{
	QObject::connect(button, &QAbstractButton::clicked, button, [=]( bool checked ) { fn(button, checked ); });
}

void	QAbstractButton_setText	(QAbstractButton* w, const char* s)	{ w->setText(QString(s)); }
void	QAbstractButton_setIcon	(QAbstractButton* w, QIcon *icon)	{ w->setIcon(*icon); }
//[cf]
//[of]:			QPushButton
QPushButton*	QPushButton_new	(const char* s, QWidget* parent)	{ return new QPushButton(QString(s), parent); }
void	QPushButton_setDefault	(QPushButton* w, int def)	{ w->setDefault(def); }
//[cf]
//[of]:			QCheckBox
QCheckBox*	QCheckBox_new	(const char* text, QWidget* parent)	{ return new QCheckBox(text, parent); }
Qt::CheckState 	QCheckBox_checkState	(QCheckBox* c)	{ return c->checkState(); }
void 	QCheckBox_setCheckState	(QCheckBox* c, Qt::CheckState state)	{ c->setCheckState(state); }

typedef void (*StateChangedFunc)(QCheckBox* w, int state);

void QCheckBox_connectStateChanged(QCheckBox* w, StateChangedFunc fn)
{
	QObject::connect(w, &QCheckBox::stateChanged, w, [=](int state) { fn(w, state); });
}
//[cf]
//[of]:		QLineEdit
typedef void (*TextChangedFunc)(QLineEdit* edit);

void QLineEdit_connectTextChanged(QLineEdit* edit, TextChangedFunc fn)
{
	QObject::connect(edit, &QLineEdit::textChanged, edit, [=]( const QString& ) { fn(edit); });
}

QLineEdit* QLineEdit_new (const char* s, QWidget* parent)
{
	return new QLineEdit(QString(s), parent);
}

void QLineEdit_setText (QLineEdit* edit, const char* text)
{
	edit->setText(QString(text));
}

const char* QLineEdit_text (QLineEdit* edit, QByteArray* array)
{
	return toUtf8(array, edit->text());
}

int	QLineEdit_isReadOnly	(QLineEdit* edit)	{ return edit->isReadOnly(); }
void	QLineEdit_setReadOnly	(QLineEdit* edit, int ro)	{ edit->setReadOnly(ro); }
int	QLineEdit_selectionStart	(QLineEdit* edit)	{ return edit->selectionStart(); }
int	QLineEdit_cursorPosition	(QLineEdit* edit)	{ return edit->cursorPosition(); }
void	QLineEdit_setSelection	(QLineEdit* edit, int start, int length)	{ edit->setSelection(start, length); }
int	QLineEdit_hasSelectedText	(QLineEdit* edit)	{ return edit->hasSelectedText(); }
			
void 	QLineEdit_clear	(QLineEdit* edit)	{ edit->clear(); }
void 	QLineEdit_copy	(QLineEdit* edit)	{ edit->copy(); }
void 	QLineEdit_cut	(QLineEdit* edit)	{ edit->cut(); }
void 	QLineEdit_paste	(QLineEdit* edit)	{ edit->paste(); }
void 	QLineEdit_redo	(QLineEdit* edit)	{ edit->redo(); }
void	QLineEdit_selectAll	(QLineEdit* edit)	{ edit->selectAll(); }
void 	QLineEdit_undo	(QLineEdit* edit)	{ edit->undo(); }
int	QLineEdit_isRedoAvailable	(QLineEdit* edit)	{ return edit->isRedoAvailable(); }
int	QLineEdit_isUndoAvailable	(QLineEdit* edit)	{ return edit->isUndoAvailable(); }
//[cf]
//[of]:		QFrame
QFrame*	QFrame_new	(QWidget* parent)	{ return new QFrame(parent); }
void	QFrame_setFrameStyle	(QFrame* frame, int style)	{ frame->setFrameStyle(style); }
//[cf]
//[of]:			QLabel
QLabel*	QLabel_new	(const char* text, QWidget *parent, Qt::WindowFlags f)	{ return new QLabel(QString(text), parent, f); }
void	QLabel_setText	(QLabel* label, const char* text)	{ label->setText(QString(text)); }
void	QLabel_setAlignment	(QLabel* label, Qt::Alignment align)	{ label->setAlignment(align); }
void	QLabel_setBuddy	(QLabel* label, QWidget* buddy)	{ label->setBuddy(buddy); }
QWidget*	QLabel_buddy	(QLabel* label)	{ return label->buddy(); }
//[cf]
//[of]:			QAbstractScrollArea
QWidget*	QAbstractScrollArea_viewport	(QAbstractScrollArea* w)	{ return w->viewport(); }
//[cf]
//[of]:				QAbstractItemView
void	QAbstractItemView_setModel	(QAbstractItemView* w, QAbstractItemModel *model)	{ w->setModel(model); }
void	QAbstractItemView_setSelectionMode	(QAbstractItemView* w, QAbstractItemView::SelectionMode mode)	{ w->setSelectionMode(mode); }
void	QAbstractItemView_setSelectionBehavior	(QAbstractItemView* w, QAbstractItemView::SelectionBehavior behavior)	{ w->setSelectionBehavior(behavior); }
void	QAbstractItemView_clearSelection	(QAbstractItemView* w)	{ w->clearSelection(); }
QItemSelectionModel*	QAbstractItemView_selectionModel	(QAbstractItemView* w)	{ return w->selectionModel(); }
void 	QAbstractItemView_setEditTriggers	(QAbstractItemView* w, QAbstractItemView::EditTriggers triggers)	{ w->setEditTriggers(triggers); }
void 	QAbstractItemView_scrollTo	(QAbstractItemView* w, QModelIndex *index, QAbstractItemView::ScrollHint hint)	{ w->scrollTo (*index, hint); }
//[cf]
//[of]:					QTableView
void 	QTableView_setColumnWidth	(QTableView* w, int column, int width)	{ w->setColumnWidth(column, width); }
QHeaderView *	QTableView_horizontalHeader	(QTableView* w)	{ return w->horizontalHeader(); }
void	QTableView_selectRow	(QTableView* w, int row)	{ w->selectRow(row); }
QHeaderView *	QTableView_verticalHeader	(QTableView* w)	{ return w->verticalHeader(); }
//[cf]
//[of]:						QTableWidget
typedef void (*ChangedFunc)(QObject* w);

QTableWidget*	QTableWidget_new	(QWidget *parent)	{ return new QTableWidget(parent); }
int 	QTableWidget_columnCount	(QTableWidget* w)	{ return w->columnCount(); }
int 	QTableWidget_currentColumn	(QTableWidget* w)	{ return w->currentColumn(); }
int 	QTableWidget_currentRow	(QTableWidget* w)	{ return w->currentRow(); }
QTableWidgetItem *	QTableWidget_item	(QTableWidget* w, int row, int column)	{ return w->item(row, column); }
int 	QTableWidget_rowCount	(QTableWidget* w)	{ return w->rowCount(); }
void 	QTableWidget_setColumnCount	(QTableWidget* w, int columns)	{ w->setColumnCount(columns); }
void 	QTableWidget_setCurrentCell	(QTableWidget* w, int row, int column)	{ w->setCurrentCell(row, column); }
void 	QTableWidget_setHorizontalHeaderItem	(QTableWidget* w, int column, QTableWidgetItem *item)	{ w->setHorizontalHeaderItem(column, item); }
void 	QTableWidget_setHorizontalHeaderLabels	(QTableWidget* w, QStringList* labels)	{ w->setHorizontalHeaderLabels(*labels); }
void 	QTableWidget_setItem	(QTableWidget* w, int row, int column, QTableWidgetItem *item)	{ w->setItem(row, column, item); }
void 	QTableWidget_setRowCount	(QTableWidget* w, int rows)	{ w->setRowCount(rows); }
void 	QTableWidget_clear	(QTableWidget* w)	{ w->clear(); }
void	QTableWidget_connectItemSelectionChanged	(QTableWidget* w, ChangedFunc fn)	{ QObject::connect(w, &QTableWidget::itemSelectionChanged, w, [=]() { fn(w); }); }
void	QTableWidget_connectItemDoubleClicked	(QTableWidget* w, ChangedFunc fn)	{ QObject::connect(w, &QTableWidget::itemDoubleClicked, w, [=]() { fn(w); }); }//[cf]
//[of]:					QHeaderView
void	QHeaderView_setStretchLastSection	(QHeaderView* w, int stretch)	{ w->setStretchLastSection(stretch); }
void	QHeaderView_resizeSections	(QHeaderView* w, QHeaderView::ResizeMode mode)	{ w->resizeSections(mode); }
void 	QHeaderView_setDefaultSectionSize	(QHeaderView* w, int size)	{ w->setDefaultSectionSize(size); }
void 	QHeaderView_setMinimumSectionSize	(QHeaderView* w, int size)	{ w->setMinimumSectionSize(size); }
void 	QHeaderView_setMaximumSectionSize	(QHeaderView* w, int size)	{ w->setMaximumSectionSize(size); }
void	QHeaderView_setSectionResizeMode	(QHeaderView* w, QHeaderView::ResizeMode mode)	{ w->setSectionResizeMode(mode); }
//[cf]
//[of]:					QTreeView
QTreeView*	QTreeView_new	(QWidget* parent)	{ return new QTreeView (parent); }
void	QTreeView_setHeaderHidden	(QTreeView* w, int hide)	{ w->setHeaderHidden(hide); }
int	QTreeView_isExpanded	(QTreeView* w, QModelIndex* index) 	{ return w->isExpanded(*index); }
void 	QTreeView_setExpandsOnDoubleClick	(QTreeView* w, int enable)	{ w->setExpandsOnDoubleClick(enable); }
void 	QTreeView_setItemsExpandable	(QTreeView* w, int enable)	{ w->setItemsExpandable(enable); }
void 	QTreeView_setRootIsDecorated	(QTreeView* w, int show)	{ w->setRootIsDecorated(show); }
void 	QTreeView_collapse	(QTreeView* w, QModelIndex* index)	{ w->collapse(*index); }
void 	QTreeView_collapseAll	(QTreeView* w)	{ w->collapseAll(); }
void 	QTreeView_expand	(QTreeView* w, QModelIndex* index)	{ w->expand(*index); }
void 	QTreeView_expandAll	(QTreeView* w)	{ w->expandAll(); }
//[cf]
//[of]:		QComboBox
QComboBox*	QComboBox_new	(QWidget* parent)	{ return new QComboBox(parent); }
void	QComboBox_setEditable	(QComboBox* combo, int e)	{ combo->setEditable(e); }
void	QComboBox_setCompleter	(QComboBox* combo, QCompleter* completer)	{ combo->setCompleter(completer); }
int	QComboBox_currentIndex	(QComboBox* combo)	{ return combo->currentIndex(); }
void	QComboBox_setCurrentIndex	(QComboBox* combo, int i)	{ return combo->setCurrentIndex(i); }
const char*	QComboBox_currentText	(QComboBox* combo, QByteArray* array)	{ return toUtf8(array, combo->currentText()); }
void	QComboBox_setCurrentText	(QComboBox* combo, const char* text)	{ combo->setCurrentText(QString(text)); }
void	QComboBox_addItem	(QComboBox* combo, const char* text)	{ combo->addItem(QString(text)); }
void	QComboBox_showPopup	(QComboBox* combo)	{ combo->showPopup(); }
void	QComboBox_hidePopup	(QComboBox* combo)	{ combo->hidePopup(); }
void	QComboBox_clear	(QComboBox* combo)	{ combo->clear(); }
QLineEdit*	QComboBox_lineEdit	(QComboBox* combo)	{ return combo->lineEdit(); }

typedef void (*CurrentIndexChangedFunc)(QComboBox* b, int index);

void QComboBox_connectCurrentIndexChanged(QComboBox* b, CurrentIndexChangedFunc fn)
{
	void (QComboBox::*f)(int) = &QComboBox::currentIndexChanged;
	QObject::connect(b, f, b, [=]( int index ) { fn(b, index); });
}

typedef void (*CurrentTextChangedFunc)(QComboBox* b);

void QComboBox_connectCurrentTextChanged(QComboBox* b, CurrentTextChangedFunc fn)
{
	QObject::connect(b, &QComboBox::currentTextChanged, b, [=]( const QString& ) { fn(b); });
}
//[cf]
//[of]:		QListWidget
QListWidget*	QListWidget_new	(QWidget* parent)	{ return new QListWidget(parent); }
void	QListWidget_clear	(QListWidget* w)	{ w->clear(); }
void 	QListWidget_addItem	(QListWidget* w, const char* label)	{ w->addItem(label); }
int 	QListWidget_currentRow	(QListWidget* w)	{ return w->currentRow(); }
void	QListWidget_setCurrentRow	(QListWidget* w, int row)	{ w->setCurrentRow(row); }
void	QListWidget_removeItem	(QListWidget* w, int row)	{ delete w->takeItem(row); }
void	QListWidget_connectItemDoubleClicked	(QListWidget* w, ChangedFunc fn)	{ QObject::connect(w, &QListWidget::itemDoubleClicked, w, [=]() { fn(w); }); }

typedef void (*CurrentRowChangedFunc)(QListWidget* w, int index);

void QListWidget_connectCurrentRowChanged(QListWidget* w, CurrentRowChangedFunc fn)
{
	void (QListWidget::*f)(int) = &QListWidget::currentRowChanged;
	QObject::connect(w, f, w, [=]( int index ) { fn(w, index); });
}
//[cf]
//[of]:		QAbstractSlider
int	QAbstractSlider_minimum	(QAbstractSlider* s)	{ return s->minimum(); }
int	QAbstractSlider_maximum	(QAbstractSlider* s)	{ return s->maximum(); }
int	QAbstractSlider_pageStep	(QAbstractSlider* s)	{ return s->pageStep(); }
int	QAbstractSlider_value	(QAbstractSlider* s)	{ return s->value(); }
void	QAbstractSlider_setMinimum	(QAbstractSlider* s, int v)	{ s->setMinimum(v); }
void	QAbstractSlider_setMaximum	(QAbstractSlider* s, int v)	{ s->setMaximum(v); }
void	QAbstractSlider_setPageStep	(QAbstractSlider* s, int v)	{ s->setPageStep(v); }
void	QAbstractSlider_setValue	(QAbstractSlider* s, int v)	{ s->setValue(v); }

typedef void (*ValueChangedFunc)(QAbstractSlider* a, int value);

void QAbstractSlider_connectValueChanged(QAbstractSlider* a, ValueChangedFunc fn)
{
	QObject::connect(a, &QAbstractSlider::valueChanged, a, [=]( int value ) { fn(a, value); });
}
//[cf]
//[of]:			QScrollBar
QScrollBar*	QScrollBar_new	(Qt::Orientation orientation, QWidget *parent)	{ return new QScrollBar(orientation, parent); }
//[cf]
//[of]:		QTabWidget
typedef void (*TabCloseRequestedFunc)(QWidget* w, int index);

QTabWidget*	QTabWidget_new	(QWidget* parent)	{ return new QTabWidget(parent); }
int 	QTabWidget_addTab	(QTabWidget* w, QWidget *page, const char* label)	{ return w->addTab(page, QString(label)); }
int 	QTabWidget_addTabWithIcon	(QTabWidget* w, QWidget *page, QIcon* icon, const char* label)	{ return w->addTab(page, *icon, QString(label)); }
void 	QTabWidget_removeTab	(QTabWidget* w, int index)	{ w->removeTab(index); }
int 	QTabWidget_currentIndex	(QTabWidget* w)	{ return w->currentIndex(); }
int 	QTabWidget_tabsClosable	(QTabWidget* w)	{ return w->tabsClosable(); }
void 	QTabWidget_setCurrentIndex	(QTabWidget* w, int index)	{ w->setCurrentIndex(index); }
void	QTabWidget_setTabsClosable	(QTabWidget* w, int closeable)	{ w->setTabsClosable(closeable); }
void 	QTabWidget_setUsesScrollButtons	(QTabWidget* w, int useButtons)	{ w->setUsesScrollButtons(useButtons); }
void	QTabWidget_setTabText	(QTabWidget* w, int index, const char* label)	{ w->setTabText(index, QString(label)); }
void	QTabWidget_connectCurrentChanged	(QTabWidget* w, ChangedFunc fn)	{ QObject::connect(w, &QTabWidget::currentChanged, w, [=]( int ) { fn(w); }); }
void	QTabWidget_connectTabCloseRequested	(QTabWidget* w, TabCloseRequestedFunc fn)	{ QObject::connect(w, &QTabWidget::tabCloseRequested, w, [=]( int index) { fn(w, index); }); }

void	QTabWidget_setTabIcon	(QTabWidget* w, int index, QIcon* icon)	
{ 
	if( icon )
		w->setTabIcon(index, *icon); 
	else
		w->setTabIcon(index, QIcon()); 
}
//[cf]
//[of]:		QDialog
QDialog*	QDialog_new	(QWidget* parent)	{ return new QDialog(parent); }
int	QDialog_exec	(QDialog* d)	{ return d->exec(); }
int	QDialog_result	(QDialog* d)	{ return d->result(); }
void	QDialog_setResult	(QDialog* d, int r)	{ d->setResult(r); }
void	QDialog_setModal	(QDialog* d, int m)	{ d->setModal(m); }
//[cf]
//[of]:			QMessageBox
int QMessageBox_question (	QWidget*	parent, 
	const char*	title, 
	const char*	text, 
	QMessageBox::StandardButtons buttons, 
	QMessageBox::StandardButton defaultButton)
{
	QString qtitle(title);
	QString qtext(text);
	return (int)QMessageBox::question (parent, qtitle, qtext, buttons, defaultButton);
}
//[cf]
//[of]:			QColorDialog
typedef unsigned int color_t;

static QColor toColor(color_t c)
{
	return QColor (	(c >> 16) & 255,
		(c >> 8) & 255,
		c & 255);
}

static color_t toInteger (const QColor& c)
{
	return (color_t)(c.blue() | (c.green() << 8) | (c.red() << 16));
}

QColorDialog*	QColorDialog_new	(color_t initial, QWidget *parent)	{ return new QColorDialog(toColor(initial), parent); }
void	QColorDialog_open	(QColorDialog* dialog, QObject *receiver, const char *member)	{ dialog->open(receiver, member); }
color_t	QColorDialog_selectedColor	(QColorDialog* dialog)	{ return toInteger(dialog->selectedColor()); }
//[cf]
//[of]:			QFileDialog

const char* QFileDialog_getSaveFileName(	QWidget*	parent, 
	const char*	caption,
	const char*	dir,
	const char*	filter, 
	QFileDialog::Options	options,
	QByteArray*	array)
{
	return toUtf8(array, QFileDialog::getSaveFileName (parent, QString(caption), QString(dir), QString(filter), NULL, options));
}

const char* QFileDialog_getOpenFileName(	QWidget*	parent, 
	const char*	caption,
	const char*	dir,
	const char*	filter, 
	QFileDialog::Options	options,
	QByteArray*	array)
{
	return toUtf8(array, QFileDialog::getOpenFileName (parent, QString(caption), QString(dir), QString(filter), NULL, options));
}

void QFileDialog_getOpenFileNames(	QWidget*	parent, 
	const char*	caption,
	const char*	dir,
	const char*	filter, 
	QFileDialog::Options	options,
	QStringList*	list)
{
	*list = QFileDialog::getOpenFileNames (parent, QString(caption), QString(dir), QString(filter), NULL, options);
}

const char* QFileDialog_getExistingDirectory(	QWidget*	parent,
	const char*	caption,
	const char*	dir,
	QFileDialog::Options	options,
	QByteArray*	array)
{
	return toUtf8(array, QFileDialog::getExistingDirectory (parent, QString(caption), QString(dir), options));
}
//[cf]
//[of]:		QMenuBar
QMenuBar*	QMenuBar_new(QWidget* parent) { return new QMenuBar(parent); }
QMenu*	QMenuBar_addMenu(QMenuBar* bar, const char* title) { return bar->addMenu(QString(title)); }//[cf]
//[of]:		QMenu
QMenu*	QMenu_new(QWidget* parent) { return new QMenu(parent); }

QMenu*	QMenu_addMenu	(QMenu* menu, const char* title)	{ return menu->addMenu(QString(title)); }
QAction*	QMenu_addAction	(QMenu* menu, const char* title)	{ return menu->addAction(QString(title)); }
QAction*	QMenu_addSeparator	(QMenu* menu)	{ return menu->addSeparator(); }
QAction*	QMenu_insertSeparator	(QMenu* menu, QAction* before)	{ return menu->insertSeparator(before); }
QAction*	QMenu_menuAction	(QMenu* menu)	{ return menu->menuAction(); }
QAction*	QMenu_exec	(QMenu* menu, int x, int y)	{ return menu->exec(QPoint(x, y)); }
//[cf]
//[of]:	QAction
typedef void (*TriggeredFunc)(QAction* a, int checked);

void QAction_connectTriggered(QAction* a, TriggeredFunc fn)
{
	QObject::connect(a, &QAction::triggered, a, [=]( bool checked ) { fn(a, checked); });
}

QAction* QAction_new (const char* title, int k1) 
{ 
	QAction* action = new QAction (QString(title)); 
	if( k1 != 0 )
	{
		QKeySequence seq(k1);
		action->setShortcut(seq);
	}
	return action;
}

int	QAction_isCheckable	(QAction* a)	{ return a->isCheckable(); }
void	QAction_setCheckable	(QAction* a, int c)	{ a->setCheckable(c); }
int	QAction_isChecked	(QAction* a)	{ return a->isChecked(); }
void	QAction_setChecked	(QAction* a, int c)	{ a->setChecked(c); }
int	QAction_isEnabled	(QAction* a)	{ return a->isEnabled(); }
void	QAction_setEnabled	(QAction* a, int e)	{ a->setEnabled(e); }
void	QAction_setShortcut	(QAction* a, int k)	{ a->setShortcut(QKeySequence(k)); }
void	QAction_setText	(QAction* a, const char* text)	{ a->setText(QString(text)); }
//[cf]
//[of]:	QClipboard
const QMimeData*	QClipboard_mimeData	(QClipboard* clipboard, QClipboard::Mode mode) { return clipboard->mimeData(mode); }
void	QClipboard_setMimeData	(QClipboard* clipboard, QMimeData* data, QClipboard::Mode mode) { clipboard->setMimeData(data, mode); }
void	QClipboard_setText	(QClipboard* clipboard, const char* text, QClipboard::Mode mode) { clipboard->setText(QString(text), mode); }
void	QClipboard_clear	(QClipboard* clipboard, QClipboard::Mode mode) { clipboard->clear(mode); }

typedef void (*DataChangedFunc)(QClipboard* clipboard);

void QClipboard_connectDataChanged(QClipboard* clipboard, DataChangedFunc fn)
{
	QObject::connect(clipboard, &QClipboard::dataChanged, clipboard, [=]() { fn(clipboard); });
}
//[cf]
//[of]:	QMimeData
QMimeData*	QMimeData_new	()	{ return new QMimeData(); }
void	QMimeData_formats	(QMimeData* data, QStringList* list)	{ *list = data->formats(); }
int	QMimeData_hasFormat	(QMimeData* data, const char* mimeType)	{ return data->hasFormat(QString(mimeType)); }
void 	QMimeData_data	(QMimeData* data, const char* mimeType, QByteArray* array)	{ *array = data->data(mimeType); }
void	QMimeData_setData	(QMimeData* data, const char* mimeType, QByteArray* array)	{ data->setData(QString(mimeType), *array); }
void	QMimeData_setText	(QMimeData* data, const char* text)	{ data->setText(QString(text)); }

void	QMimeData_getFiles	(QMimeData* data, QStringList* list)
{
	QList<QUrl> urls = data->urls();
	for(int i=0, n=urls.size(); i<n; i++)
		list->append(urls.at(i).toLocalFile());
}
//[cf]
//[of]:	QTimer
QTimer*	QTimer_new	()	{ return new QTimer; }
void	QTimer_setInterval	(QTimer* timer, int msec)	{ timer->setInterval(msec); }
void	QTimer_start	(QTimer* timer)	{ timer->start(); }
void	QTimer_stop	(QTimer* timer)	{ timer->stop(); }

typedef void (*TimeoutFunc)(QTimer* timer);

void QTimer_connectTimeout(QTimer* timer, TimeoutFunc fn)
{
	QObject::connect(timer, &QTimer::timeout, timer, [=]() { fn(timer); });
}
//[cf]
//[of]:	QStyle
QIcon*	QStyle_newStandardIcon(QStyle* style, QStyle::StandardPixmap standardIcon)	{ return new QIcon(style->standardIcon(standardIcon));  }
//[cf]
//[of]:	QItemSelectionModel
typedef void (*CurrentChangedFunc)(QObject* w, const QModelIndex* current, const QModelIndex* previous);

int QItemSelectionModel_selectionRow	(QItemSelectionModel* s) 
{
	const QItemSelection selection = s->selection();
	QModelIndexList indexes = selection.indexes();
	return ( indexes.count() > 0 ) ? indexes.at(0).row() : -1;
}

QModelIndex* QItemSelectionModel_selectionIndex	(QItemSelectionModel* s)
{
	const QItemSelection selection = s->selection();
	QModelIndexList indexes = selection.indexes();
	return ( indexes.count() > 0 ) ? new QModelIndex(indexes.at(0)) : new QModelIndex();
}

void	QItemSelectionModel_select	(QItemSelectionModel* s, QModelIndex* index, QItemSelectionModel::SelectionFlags command)	{ s->select(*index, command); }
void	QItemSelectionModel_setCurrentIndex	(QItemSelectionModel* s, QModelIndex* index, QItemSelectionModel::SelectionFlags command)	{ s->setCurrentIndex(*index, command); }
void	QItemSelectionModel_connectCurrentChanged	(QItemSelectionModel* s, CurrentChangedFunc fn)	{ QObject::connect(s, &QItemSelectionModel::currentChanged, s, [=](const QModelIndex &current, const QModelIndex &previous) { fn(s, &current, &previous); }); }
//[cf]
//[of]:	QAbstractItemModel
void	QAbstractItemModel_index	(QAbstractItemModel* model, int row, int column, QModelIndex* parent, QModelIndex* index)	{ *index = model->index(row, column, *parent); }
//void	QAbstractItemModel_rowsInserted	(QAbstractItemModel* model, QModelIndex* parent, int first, int last)	{ emit model->rowsInserted(*parent, first, last); }
//void	QAbstractItemModel_rowsRemoved	(QAbstractItemModel* model, QModelIndex* parent, int first, int last)	{ emit model->rowsRemoved(*parent, first, last); }

void QAbstractItemModel_textChanged (QAbstractItemModel* model, QModelIndex* index)
{
	QVector<int> roles ({Qt::DisplayRole});
	emit model->dataChanged(*index, *index, roles);
}
//[cf]
//[of]:		TreeModelBridge
typedef void TreeItem;

struct TreeInterface
{
	void*	(*onTreeItem)	(void*, int, const QModelIndex*);
	void*	(*onTreeParent)	(void*, int*, const QModelIndex*);
	int	(*onTreeRowCount)	(void*, const QModelIndex*);
	const char*	(*onTreeText)	(void*, const QModelIndex*);
	QPixmap*	(*onTreeIcon)	(void*, const QModelIndex*);
	color_t	(*onTreeColor)	(void*, const QModelIndex*);
	Qt::ItemFlags	(*onTreeFlags)	(void*, const QModelIndex*);
	int	(*onTreeHasChildren)	(void*, const QModelIndex*);
};

TreeModelBridge::TreeModelBridge(void* _receiver, TreeInterface*	_interface)
{
	receiver	= _receiver;
	interface	= _interface;
}

QVariant TreeModelBridge::data(const QModelIndex &index, int role) const
{
	if( !index.isValid() )
		return QVariant();
	
	if( role == Qt::DisplayRole )
		return QVariant(interface->onTreeText(receiver, &index));
	else if( role == Qt::DecorationRole )
	{
		const QPixmap* pix = interface->onTreeIcon(receiver, &index);
		if( pix )
			return QVariant(*pix);
		else
			return QVariant();
	}
	else if( role == Qt::ForegroundRole )
	{
		color_t c = interface->onTreeColor(receiver, &index);
		if( c == 0xFFFFFFFF )
			return QVariant();
		else
			return QVariant(toColor(c));
	}
	else
		return QVariant();
}

Qt::ItemFlags TreeModelBridge::flags(const QModelIndex &index) const
{
	if( !index.isValid() )
		return Qt::NoItemFlags;
		
	return interface->onTreeFlags(receiver, &index);
}

QModelIndex TreeModelBridge::index(int row, int column, const QModelIndex &parent) const
{
	if (!hasIndex(row, column, parent))
		return QModelIndex();
	
	TreeItem* item = interface->onTreeItem(receiver, row, &parent);
	if( item )
		return createIndex(row, column, item);
	return QModelIndex();
}

QModelIndex TreeModelBridge::parent(const QModelIndex &index) const
{
	if( !index.isValid() )
		return QModelIndex();
	
	int row = 0;
	TreeItem* parent = interface->onTreeParent(receiver, &row, &index);
	if( parent )
	    	return createIndex(row, 0, parent);
	return QModelIndex();
}

int TreeModelBridge::rowCount(const QModelIndex &parent) const
{
	return interface->onTreeRowCount(receiver, &parent);
}

int TreeModelBridge::columnCount(const QModelIndex &) const
{
	return 1;
}

bool TreeModelBridge::hasChildren(const QModelIndex &parent) const
{
	// The root has children
	if( !parent.isValid() )
		return true;
		
	return interface->onTreeHasChildren(receiver, &parent);
}

void TreeModelBridge::beginReplace (QModelIndex* parent, int first0, int inserted0, int deleted0)
{
	int replaced = std::min(inserted0, deleted0);
	int first = first0 + replaced;
	int inserted = inserted0 - replaced;
	int deleted = deleted0 - replaced;

	if( deleted > 0 )
		beginRemoveRows(*parent, first, first + deleted - 1);
	else if( inserted > 0 )
		beginInsertRows(*parent, first, first + inserted - 1);
}

void TreeModelBridge::endReplace (QModelIndex* parent, int first0, int inserted0, int deleted0)
{
	int replaced = std::min(inserted0, deleted0);
	int first = first0 + replaced;
	int inserted = inserted0 - replaced;
	int deleted = deleted0 - replaced;

	if( first > first0 )
	{
		QModelIndex top = index(first0, 0, *parent);
		QModelIndex bottom = index(first - 1, 0, *parent);
		dataChanged(top, bottom);
	}
	
	if( deleted > 0 )
		endRemoveRows();
	else if( inserted > 0 )
		endInsertRows();
}

//[c]
TreeModelBridge* TreeModelBridge_new (void* receiver, TreeInterface* interface) { return new TreeModelBridge(receiver, interface); }

void	TreeModelBridge_beginReplace	(TreeModelBridge* model, QModelIndex* parent, int first, int inserted, int deleted)	{ model->beginReplace(parent, first, inserted, deleted); }
void	TreeModelBridge_endReplace	(TreeModelBridge* model, QModelIndex* parent, int first, int inserted, int deleted)	{ model->endReplace(parent, first, inserted, deleted); }
//[cf]

//[of]:QStringList
QStringList*	QStringList_new	()	{ return new QStringList; }
void	QStringList_delete	(QStringList* list)	{ delete list; }
int	QStringList_count	(QStringList* list)	{ return list->count(); }
const char*	QStringList_value	(QStringList* list, int index, QByteArray* array)	{ return toUtf8(array, list->value(index)); }
void 	QStringList_append	(QStringList* list, const char* value)	{ list->append(QString(value)); }
//[cf]
//[of]:QByteArray
QByteArray*	QByteArray_new	()	{ return new QByteArray(); }
QByteArray*	QByteArray_newFromRawData	(const char *data, int size)	{ return new QByteArray(data, size); }
void	QByteArray_delete	(QByteArray* array)	{ delete array; }
const char*	QByteArray_constData	(QByteArray* array)	{ return array->constData(); }
int	QByteArray_size	(QByteArray* array)	{ return array->size(); }
//[cf]
//[of]:QEvent
int	QEvent_registerEventType	()	{ return QEvent::registerEventType(); }
QEvent*	QEvent_new	(QEvent::Type type)	{ return new QEvent(type); }
int	QEvent_type	(QEvent* event)	{ return event->type(); }
void	QEvent_ignore	(QEvent* event)	{ event->ignore(); }
void	QEvent_accept	(QEvent* event)	{ event->accept(); }
//[cf]
//[of]:	QInputEvent
int QInputEvent_modifiers(QInputEvent* e) { return e->modifiers(); }
//[cf]
//[of]:		QKeyEvent
int	QKeyEvent_key	(QKeyEvent* e)	{ return e->key(); }
const char*	QKeyEvent_text	(QKeyEvent* e, QByteArray* array)	{ return toUtf8(array, e->text()); }
//[cf]
//[of]:		QMouseEvent
int QMouseEvent_x	(QMouseEvent* e)	{ return e->x(); }
int QMouseEvent_y	(QMouseEvent* e)	{ return e->y(); }
int QMouseEvent_button	(QMouseEvent* e)	{ return e->button(); }
int QMouseEvent_flags	(QMouseEvent* e)	{ return e->flags(); }
//[cf]
//[of]:		QWheelEvent
int QWheelEvent_angleDeltaX	(QWheelEvent* e) { return e->angleDelta().x(); }
int QWheelEvent_angleDeltaY	(QWheelEvent* e) { return e->angleDelta().y(); }
int QWheelEvent_x	(QWheelEvent* e) { return e->x(); }
int QWheelEvent_y	(QWheelEvent* e) { return e->y(); }
//[cf]
//[of]:		QContextMenuEvent
int QContextMenuEvent_x	(QContextMenuEvent* e)	{ return e->x(); }
int QContextMenuEvent_y	(QContextMenuEvent* e)	{ return e->y(); }
int QContextMenuEvent_globalX	(QContextMenuEvent* e)	{ return e->globalX(); }
int QContextMenuEvent_globalY	(QContextMenuEvent* e)	{ return e->globalY(); }
//[cf]
//[of]:	QInputMethodEvent
const char* QInputMethodEvent_commitString (QInputMethodEvent*e, QByteArray* array)	{ return toUtf8(array, e->commitString()); }//[cf]
//[of]:	QPaintEvent
void	QPaintEvent_rect	(QPaintEvent* e, rectangle_t* r)	{ r->set(e->rect()); }


//[cf]
//[of]:	QDropEvent
const QMimeData*	QDropEvent_mimeData	(QDropEvent* e)	{ return e->mimeData(); }
int	QDropEvent_x	(QDropEvent* e)	{ return e->pos().x(); }
int	QDropEvent_y	(QDropEvent* e)	{ return e->pos().y(); }
void 	QDropEvent_acceptProposedAction	(QDropEvent* e)	{ e->acceptProposedAction(); }
//[cf]
//[of]:	QUserEvent
class QUserEvent : public QEvent
{
public:
	void* data;

	QUserEvent(QEvent::Type type, void* d) : 
		QEvent(type),
		data(d)
	{
	}
};

QUserEvent*	QUserEvent_new	(QEvent::Type type, void* data)	{ return new QUserEvent(type, data); }
void*	QUserEvent_data	(QUserEvent* e)	{ return e->data; }
//[cf]
//[of]:EventFilter
EventFilter* EventFilter_new (int (*fn)(QObject *obj, QEvent *event))
{
	return new EventFilter(fn);
}

void EventFilter_delete (EventFilter* filter)
{
	delete filter;
}

EventFilter::EventFilter(int (*fn)(QObject *obj, QEvent *event))
{
	this->fn = fn;
}

bool EventFilter::eventFilter(QObject *obj, QEvent *event)
{
	if( fn(obj, event) )
		return true;
	else
		// Default processing
		return QObject::eventFilter(obj, event);
}
//[cf]

//[of]:QFont
QFont*	QFont_newDefault	()	{ return new QFont(); }
QFont*	QFont_new	(const char* family, int pointSize, int weight, int italic)	{ return new QFont(family, pointSize, weight, italic); }
const char*	QFont_substitute	(const char* familyName, QByteArray* array)	{ return toUtf8(array, QFont::substitute(QString(familyName))); }
void	QFont_delete	(QFont* font)	{ delete font; }
int	QFont_pointSize	(QFont* font)	{ return font->pointSize(); }
int	QFont_exactMatch	(QFont* font)	{ return font->exactMatch(); }
const char*	QFont_family	(QFont* font, QByteArray* array)	{ QFontInfo info(*font); return toUtf8(array, info.family()); }
void	QFont_setKerning	(QFont* font, int en)	{ font->setKerning(en); }
void	QFont_setHintingPreference	(QFont* font, QFont::HintingPreference p)	{ font->setHintingPreference(p); }
void	QFont_setStyleStrategy	(QFont* font, QFont::StyleStrategy s)	{ font->setStyleStrategy(s); }
//[cf]
//[of]:QFontMetrics
QFontMetrics*	QFontMetrics_new	(QFont* font)	{ return new QFontMetrics(*font); }
void	QFontMetrics_delete	(QFontMetrics* fm)	{ delete fm; }
int	QFontMetrics_ascent	(QFontMetrics* fm)	{ return fm->ascent(); }
int	QFontMetrics_descent	(QFontMetrics* fm)	{ return fm->descent(); }
int	QFontMetrics_height	(QFontMetrics* fm)	{ return fm->height(); }	// ascent + descent + 1 !
int	QFontMetrics_averageCharWidth	(QFontMetrics* fm)	{ return fm->averageCharWidth(); }
int	QFontMetrics_maxWidth	(QFontMetrics* fm)	{ return fm->maxWidth(); }
void	QFontMetrics_boundingRect	(QFontMetrics* fm, int x, int y, int w, int h, int flags, const char* text, int len, rectangle_t* r) { r->set(fm->boundingRect(x, y, w, h, flags, QString::fromUtf8(text, len))); }
int	QFontMetrics_width	(QFontMetrics* fm, const char* text, int len) { return fm->width(QString::fromUtf8(text, len)); }
//[cf]
//[of]:QCursor
QCursor*	QCursor_new	(Qt::CursorShape shape)	{ return new QCursor(shape); }
void	QCursor_delete	(QCursor* cursor)	{ delete cursor; }
void	QCursor_pos	(point_t* pt)	{ pt->set(QCursor::pos()); }
//[cf]
//[of]:QPalette
QPalette*	QPalette_new	()	{ return new QPalette(); }
void	QPalette_delete	(QPalette* palette)	{ delete palette; }
color_t	QPalette_color	(QPalette* palette, QPalette::ColorGroup group, QPalette::ColorRole role)	{ return toInteger(palette->color(group, role)); }
void	QPalette_setColor	(QPalette* palette, QPalette::ColorGroup group, QPalette::ColorRole role, color_t c)	{ palette->setColor(group, role, toColor(c)); }
//[cf]
//[of]:QPainter
QPainter*	QPainter_newOnWidget	(QWidget* widget)	{ return new QPainter (widget); }
void	QPainter_delete	(QPainter* painter)	{ delete painter; }
void	QPainter_fillRect	(QPainter* painter, int x, int y, int w, int h, color_t c)	{ painter->fillRect(x, y, w, h, toColor(c)); }
void	QPainter_setFont	(QPainter* painter, QFont* font)	{ painter->setFont(*font); }
void	QPainter_setBackgroundMode	(QPainter* painter, Qt::BGMode mode)	{ painter->setBackgroundMode(mode); }
void	QPainter_setPen	(QPainter* painter, color_t c)	{ painter->setPen(toColor(c)); }
void	QPainter_drawText	(QPainter* painter, int x, int y, int w, int h, int flags, const char* text, int len)	{ painter->drawText(x, y, w, h, flags, QString::fromUtf8(text, len)); }
void	QPainter_drawPixmap	(QPainter* painter, int x, int y, QPixmap* pixmap)	{ painter->drawPixmap(x, y, *pixmap); }
void	QPainter_boundingRect	(QPainter* painter, int x, int y, int w, int h, int flags, const char* text, int len, rectangle_t* r) { r->set(painter->boundingRect(x, y, w, h, flags, QString::fromUtf8(text, len))); }

void QPainter_size (QPainter* painter, const char* text, int len, int* w, int* h) { 
	QFontMetrics fm = painter->fontMetrics();
	*w = fm.width(QString::fromUtf8(text, len)); 
	*h = fm.height();
}
//[cf]
//[of]:QPixmap
QPixmap*	QPixmap_new	()	{ return new QPixmap; }
void	QPixmap_delete	(QPixmap* p)	{ delete p; }
int	QPixmap_loadFromData	(QPixmap* p, const uchar *data, uint len, const char *format)	{ return p->loadFromData(data, len, format); }
//[cf]
//[of]:QIcon
QIcon*	QIcon_new	(QPixmap* p)	{ return new QIcon(*p); }
QIcon*	QIcon_newFromTheme	(const char* name)	{ return new QIcon(QIcon::fromTheme(QString(name))); }
void	QIcon_delete	(QIcon* i)	{ delete i; }
//[cf]
//[of]:QTableWidgetItem
QTableWidgetItem*	QTableWidgetItem_new	(const char* text, int type)	{ return new QTableWidgetItem(QString(text), type); }
void 	QTableWidgetItem_setText	(QTableWidgetItem* item, const char* text)	{ item->setText(QString(text)); }
void 	QTableWidgetItem_setTextAlignment	(QTableWidgetItem* item, Qt::Alignment alignment)	{ item->setTextAlignment(alignment); }
//[cf]
//[of]:QModelIndex
QModelIndex*	QModelIndex_new	()	{ return new QModelIndex(); }
void	QModelIndex_delete	(QModelIndex* index)	{ delete index; }
int 	QModelIndex_column	(QModelIndex* index)	{ return index->column(); }
void *	QModelIndex_internalPointer	(QModelIndex* index)	{ return index->internalPointer(); }
int	QModelIndex_isValid	(QModelIndex* index)	{ return index->isValid(); }
void	QModelIndex_parent	(QModelIndex* index, QModelIndex* out)	{ *out = index->parent(); }
int 	QModelIndex_row	(QModelIndex* index)	{ return index->row(); }
//[cf]

}
